%carnot
% Plots a Carnot heat cycle (volume, pressure) diagram.
% This is the most efficient heat engine possible using
% energy exchange between a hot reservoir at temperature T_H and a cold
% reservoir at temperature T_C. The engine is assumed to contain an ideal
% gas.
%
% LAST UPDATED by Andy French 16/10/15
%
% Ref: https://en.wikipedia.org/wiki/Carnot_cycle
%
% The Carnot Cycle p,V diagram
%
% 1: Isothermal expansion of gas at T_H, which does work on the
%    surroundings. The expansion is powered by absorbtion of heat Q_in from
%    the hot reservoir.
% 2: Isentropic (i.e. adiabatic or no heat change) expansion of gas, until
%    gas temperature reaches T_C
% 3: Isothermal compression of gas at T_C. Surroundings do work on the gas,
%    which equates to the heat Q_out extracted from the gas.
% 4: Isentropic compression of the gas, until it reaches temperature T_H
%
% The total work done by the gas on the surroundings is the cyclic integral
% of of pressure with respect to volume.
%
% The net heat input to the system during the cycle is Q_in - Q_out 
%
% By the first law of thermodynamics, the work done by the gas during the
% cycle must equal the net heat input. This is because the temperature of
% the gas at the start and end of the cycle is the same (T_H) and the
% internal energy of an ideal gas (U) is a function of temeparture only.

function carnot

%%%%%%%%%%% INPUTS %%%%%%%%%%%%%%%%%%%

%Hot reservoir temperature in degrees Celcius
T_H = 90;

%Cold reservoir temperature in degrees Celcius
T_C = 20;

%Molar mass of gas /gmol^-1
M = 28.966;

%Mass of gas /kg
m = 1;

%Specific heat capacity 'alpha parameter' i.e. number of degrees of freedom
%of molecular motion. A non integer number can be used as an empirical
%correction to make a more exact specific heat capacity.
alpha = 3;

%Volume of gas in m^3 at various stages in the cycle. Note V3 & V4 are derived
%from V1, V2 to enable the cycle to be closed.
V1 = 0.1;
V2 = 2*V1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of data points between cycle points
N = 200;

%Fontsize
fsize = 18;

%Molar gas constant /Jmol^-1K^-1
R = 8.314;

%Pa per atmosphere
atm = 101325;

%Calculate temperatures in Kelvin
T_H = T_H + 273;
T_C = T_C + 273;

%Gamma parameter (ratio of isobaric and isochoric specific heat capacities)
y = 1 + 2/alpha;

%Number of moles of gas
n = m/(M*1e-3);

%Volume /m^3 at position 3
V3 = V2 * (T_H/T_C)^(1/(y-1));

%Volume /m^3 at position 4
V4 = V1 * (T_H/T_C)^(1/(y-1));

%Heat input /J
Q_in = n*R*T_H*log( V2/V1 );

%Heat output /J
Q_out = n*R*T_C*log( V3/V4 );

%Total work done by the ideal gas on the surroundings /J
W_12 = n*R*T_H*log( V2/V1 );
W_23 = ( n*R*T_H/(y-1) )*( 1 - (V2/V3)^(y-1) );
W_34 = -n*R*T_C*log( V3/V4 );
W_41 = -( n*R*T_C/(y-1) )*( (V4/V1)^(y-1) - 1);
W = W_12 + W_23 + W_34 + W_41;

%Determine entropy change /JK^-1 in the isothermal stages. (No entropy
%change in the isentropic stages). In each of the isothermal stages the
%heat added/lost is proportional to temperature, so therefore the entropy
%change is the same in each stage.
delta_S = n*R*log(V2/V1);

%Efficiency of heat engine
nu = W/Q_in;
nu_th = 1 - T_C/T_H;

%Determine volume /m^3 and pressure /Pa values

%Positions 1 to 2
V12 = linspace(V1,V2,N);
p1 = n*R*T_H/V1;
p12 = n*R*T_H./V12;

%Positions 2 to 3
V23 = linspace(V2,V3,N);
p2 = n*R*T_H/V2;
p23 = p2*(V2./V23).^y;

%Positions 3 to 4
V34 = linspace(V3,V4,N);
p3 = n*R*T_C/V3;
p34 = n*R*T_C./V34;

%Positions 4 to 1
V41 = linspace(V4,V1,N);
p4 = n*R*T_C/V4;
p41 = p4*((V4./V41).^y);

%Convert pressures to atmospheres /atm
p12 = p12/atm;
p23 = p23/atm;
p34 = p34/atm;
p41 = p41/atm;
p1 = p1/atm;
p2 = p2/atm;
p3 = p3/atm;
p4 = p4/atm;

%Convert volumes to relative values to V1
V12 = V12/V1;
V23 = V23/V1;
V34 = V34/V1;
V41 = V41/V1;

%Convert temperatures back to degrees
T_H = T_H - 273;
T_C = T_C - 273;

%Plot p,V diagram and save PNG
figure('name','Carnot heat cycle','color',[1,1,1] )
plot(V12,p12,'r-',V23,p23,'g',V34,p34,'b-',V41,p41,'k');
legend({'Isothermal expansion','Isentropic expansion',...
    'Isothermal compression','Isentropic compression'},'fontsize',fsize)
xlabel('Volume V /V_1','fontsize',fsize);
ylabel('Pressure p /atm','fontsize',fsize);
title({['Carnot cycle: T_H = ',num2str(T_H),'^oC, T_C = ',...
    num2str(T_C),'^oC, V_1 = ',num2str(V1),'m^3'],...
    ['W = ',num2str(W/1e3),...
    'kJ, Q_{in} = ',num2str(Q_in/1e3),...
    'kJ, Efficiency = W/Q_{in} = ',num2str(100*nu),'%']},'fontsize',fsize)
grid on;
hold on;
plot(1,p1,'r*');
plot(V2/V1,p2,'g*');
plot(V3/V1,p3,'b*');
plot(V4/V1,p4,'k*');
set(gca,'fontsize',fsize);
print( gcf, 'carnot.png','-dpng','-r300' );
cla;

%Plot T,S diagram and save PNG
plot([0,delta_S],[T_H,T_H],'r-',[delta_S,delta_S],[T_H,T_C],'g',...
    [delta_S,0],[T_C,T_C],'b-',[0,0],[T_C,T_H],'k');
legend({'Isothermal expansion','Isentropic expansion',...
    'Isothermal compression','Isentropic compression'},'fontsize',fsize,'location','best')
xlabel('Entropy change of ideal gas /JK^{-1}','fontsize',fsize);
ylabel('Temperature /^oC','fontsize',fsize);
title({['Carnot cycle: T_H = ',num2str(T_H),'^oC, T_C = ',...
    num2str(T_C),'^oC, V_1 = ',num2str(V1),'m^3'],...
    ['W = ',num2str(W/1e3),...
    'kJ, Q_{in} = ',num2str(Q_in/1e3),...
    'kJ, Efficiency = W/Q_{in} = ',num2str(100*nu),'%']},'fontsize',fsize)
xlim([-0.1*delta_S,delta_S*1.1])
ylim([0,1.1*T_H])
grid on;
hold on;
set(gca,'fontsize',fsize);
print( gcf, 'carnot S vs T.png','-dpng','-r300' );
close(gcf);

%End of code