%otto
% Plots a heat cycle (volume, pressure) diagram for an Otto cycle.
%
% The Otto Cycle is the basis of spark-ignition piston engine, 
% which is essentially how a typical petrol driven engine operates.
%
% Positions 0-1:
% Air is drawn into piston/cylinder arrangement at constant pressure.
% 
% Process 12:
% Adiabatic (isentropic) compression of the air via a piston.
% 
% Process 23: 
% Constant-volume heat transfer to the working gas from an
% external source while the piston is at maximum compression. This process
% is intended to represent the ignition of the fuel-air mixture and the
% subsequent rapid burning.
% 
% Process 34: 
% Adiabatic (isentropic) expansion (power stroke).
% 
% Process 41:
% Constant-volume process in which heat is rejected from the
% air while the piston is at maximum expansion.
% 
% Process 10:
% Air is released to the atmosphere at constant pressure.
%
% The total work done by the gas on the surroundings is the cyclic integral
% of of pressure with respect to volume i.e. the area enclosed by the
% p,V 'curve' which marks out the (p,V) values of the ideal gas
% during the cycle.
%
% LAST UPDATED by Andy French 26/10/15

function otto

%%%%%%%%%%% INPUTS %%%%%%%%%%%%%%%%%%%

%Tempearture of gas drawn into the piston /deg C
T1 = 20;

%High pressure state /atm
p3 = 100;

%Low pressure state /atm
p1 = 1;

%Volume of uncompressed gas /m^3
V1 = 1/1000;

%Volume of compressed gas /m^3
V2 = V1/10;

%Molar mass of gas /gmol^-1
M = 28.966;

%Specific heat capacity 'alpha parameter' i.e. number of degrees of freedom
%of molecular motion. A non integer number can be used as an empirical
%correction to make a more exact specific heat capacity.
alpha = 3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of data points between cycle points
N = 200;

%Fontsize
fsize = 16;

%Molar gas constant /Jmol^-1K^-1
R = 8.314;

%Gamma parameter (ratio of isobaric and isochoric specific heat capacities)
y = 1 + 2/alpha;

%Convert molar mass to kg/mol
M = M*1e-3;

%Constant volume specific heat capacity /Jkg^-1K^-1
cV = 0.5*alpha*R/M;

%Constant pressure specific heat capacity /Jkg^-1K^-1
cP = y*cV;

%Pa per atmosphere
atm = 101325;

%Convert pressures to Pa
p1 = p1*atm;
p3 = p3*atm;

%Number of moles of gas
T1 = T1 + 273;
n = p1*V1/(R*T1);

%Work out pressures at positions 2 and 4
p2 = p1*(V1/V2)^y;
p4 = p3*(V2/V1)^y;

%Work out pressures and volumes throughout the cycle
V12 = linspace(V1,V2,N);
p12 = p1*((V1./V12).^y);
V23 = linspace(V2,V2,N);
p23 = linspace(p2,p3,N);
V34 = linspace(V2,V1,N);
p34 = p3*((V2./V34).^y);
V41 = linspace(V1,V1,N);
p41 = linspace(p4,p1,N);

%Calculate temperatures in Kelvin at points 1-4 in the cycle
T2 = p2*V2/(n*R);
T3 = p3*V2/(n*R);
T4 = p4*V1/(n*R);

%Heat inputs and outputs /J
Q12 = 0;
Q23 = n*M*cV*(T3-T2);
Q34 = 0;
Q41 = n*M*cV*(T4-T1);
Q_in = Q23;
Q_out = Q41;

%Total work done
W12 = ((p1*V1)/(y-1))*(1 - (V1/V2)^(y-1));
W34 = ((p3*V2)/(y-1))*(1 - (V2/V1)^(y-1));
W = W12 + W34;

%Check conservation of energy
energy_check = W + Q_out - Q_in;

%Heat engine efficiency
r = V1/V2;
nu = W/Q_in;
nu_th = 1 - 1/(r^(y-1));
nu_carnot = 1 - min([T1,T2,T3,T4])/max([T1,T2,T3,T4]);

%Determine entropy changes /JK^-1
delta_S12 = 0;
delta_S23 = n*M*cV*log(T3/T2);
delta_S34 = 0;
delta_S41 = n*M*cV*log(T1/T4);

%Determine values of S,T plot
T12 = linspace(T1,T2,N);
S12 = zeros(1,N);
T23 = linspace(T2,T3,N);
S23 = delta_S12 + n*M*cV*log(T23/T2);
T34 = linspace(T3,T4,N);
S34 = delta_S12 + delta_S23 + zeros(1,N);
T41 = linspace(T4,T1,N);
S41 = delta_S12 + delta_S23 + delta_S34 + n*M*cV*log(T41/T4);

%Check total entropy change of gas in the cycle is zero
delta_S_check = delta_S12 + delta_S23 + delta_S34 + delta_S41;

%Convert temperatures to deg Celsius and pressures to atmospheres
p1 = p1/atm;
p2 = p2/atm;
p3 = p3/atm;
p4 = p4/atm;
p12 = p12/atm;
p23 = p23/atm;
p34 = p34/atm;
p41 = p41/atm;
T1 = T1 - 273;
T2 = T2 - 273;
T3 = T3 - 273;
T4 = T4 - 273;
T12 = T12 - 273;
T23 = T23 - 273;
T34 = T34 - 273;
T41 = T41 - 273;

%Plot p,V diagram and save PNG
figure('name','Otto heat cycle','color',[1,1,1] )
plot(V12,p12,'r-',V23,p23,'g',V34,p34,'b-',V41,p41,'k');
legend({'Adiabatic compression','Isochoric heating',...
    'Adiabatic expansion','Isochoric cooling'},'fontsize',fsize,'location','best')
xlabel('Volume V /m^3','fontsize',fsize);
ylabel('Pressure p /atm','fontsize',fsize);
title({'Otto cycle',['T_1 = ',num2str(T1),'^oC, T_2 = ',...
    num2str(T2),'^oC, T_3 = ',num2str(T3),'^oC, T_4 = ',num2str(T4),'^oC'],...
    ['W = ',num2str(W/1e3),'kJ, Q_{in} = ',num2str(Q_in/1e3),...
    'kJ, Efficiency = W/Q_{in} = ',num2str(100*nu),'%']},'fontsize',fsize)
grid on;
hold on;
plot(V1,p1,'r*');
plot(V2,p2,'g*');
plot(V2,p3,'b*');
plot(V1,p4,'k*');
xlim([V2-0.1*V1,1.1*V1]);
ylim([p1-0.1*p3,1.1*p3])
set(gca,'fontsize',fsize);
print( gcf, 'otto pV.png','-dpng','-r300' );
cla;

%Plot T,S diagram and save PNG
plot(S12,T12,'r-',S23,T23,'g',...
    S34,T34,'b-',S41,T41,'k');
legend({'Adiabatic compression','Isochoric heating',...
    'Adiabatic expansion','Isochoric cooling'},'fontsize',fsize,'location','best')
xlabel('Entropy change of ideal gas /JK^{-1}','fontsize',fsize);
ylabel('Temperature /^oC','fontsize',fsize);
title({'Otto Cycle',['T_1 = ',num2str(T1),'^oC, T_2 = ',...
    num2str(T2),'^oC, T_3 = ',num2str(T3),'^oC, T_4 = ',num2str(T4),'^oC'],...
    ['W = ',num2str(W/1e3),'kJ, Q_{in} = ',num2str(Q_in/1e3),...
    'kJ, Efficiency = W/Q_{in} = ',num2str(100*nu),'%']},'fontsize',fsize)
axis auto
grid on;
hold on;
set(gca,'fontsize',fsize);
print( gcf, 'otto S vs T.png','-dpng','-r300' );
close(gcf);

%End of code