%rectangular_pV
% Plots a 'rectangular p,V' heat cycle (volume, pressure) diagram.
%
% LAST UPDATED by Andy French 18/10/15
%
% A 'rectangular' p,V diagram
%
% 1: Isobaric (constant pressure) expansion of an ideal gas at pressure p1
%    from volume V1 to volume V2.
% 2: Isochoric (constant volume) cooling, reducing gas pressure from p1 to p3.
% 3: Isobaric compression at pressure p3 from volume V2 to volume V1.
% 4: Isochoric heating of gas, increasing its pressure from p3 to p1.
%
% The total work done by the gas on the surroundings is the cyclic integral
% of of pressure with respect to volume i.e. the area enclosed by the
% rectangular p,V 'curve' which marks out the (p,V) values of the ideal gas
% during the cycle.

function rectangular_pV

%%%%%%%%%%% INPUTS %%%%%%%%%%%%%%%%%%%

%Temperature of high pressure state /deg Celsius
T1 = 300;

%High pressure state /atm
p1 = 2;

%Low pressure state /atm
p3 = 1;

%Volume of compressed gas /m^3
V1 = 0.5;

%Volume of uncompressed gas /m^3
V2 = 1;

%Molar mass of gas /gmol^-1
M = 28.966;

%Specific heat capacity 'alpha parameter' i.e. number of degrees of freedom
%of molecular motion. A non integer number can be used as an empirical
%correction to make a more exact specific heat capacity.
alpha = 3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Number of data points between cycle points
N = 200;

%Fontsize
fsize = 16;

%Molar gas constant /Jmol^-1K^-1
R = 8.314;

%Convert molar mass to kg/mol
M = M*1e-3;

%Pa per atmosphere
atm = 101325;

%Convert pressures to Pa
p1 = p1*atm;
p3 = p3*atm;

%Calculate temperatures in Kelvin at points 2-4 in the cycle
T1 = T1 + 273;
T2 = T1*V2/V1;
T3 = T2*p3/p1;
T4 = T3*V1/V2;

%Number of moles of gas
n = p1*V1/(R*T1);

%Constant volume specific heat capacity /Jkg^-1K^-1
cV = 0.5*alpha*R/M;

%Gamma parameter (ratio of isobaric and isochoric specific heat capacities)
y = 1 + 2/alpha;

%Constant pressure specific heat capacity /Jkg^-1K^-1
cP = y*cV;

%Heat inputs and outputs /J
Q12 = n*M*cP*(T2-T1);
Q23 = n*M*cV*(T2-T3);
Q34 = n*M*cP*(T3-T4);
Q41 = n*M*cV*(T1-T4);
Q_in = Q12 + Q41;
Q_out = Q23 + Q34;

%Total work done
W = (p1-p3)*(V2-V1);

%Check conservation of energy
energy_check = W + Q_out - Q_in;

%Heat engine efficiency
nu = W/Q_in;
nu_th = ( (1/(y-1))*( y*p1/(p1-p3) + V1/(V2-V1) ) )^-1;
nu_carnot = 1 - min([T1,T2,T3,T4])/max([T1,T2,T3,T4]);

%Determine entropy changes /JK^-1
delta_S12 = n*M*cP*log(T2/T1);
delta_S23 = n*M*cV*log(T3/T2);
delta_S34 = n*M*cP*log(T4/T3);
delta_S41 = n*M*cV*log(T1/T4);

%Determine values of S,T plot
T12 = linspace(T1,T2,N);
S12 = n*M*cP*log(T12/T1);
T23 = linspace(T2,T3,N);
S23 = delta_S12 + n*M*cV*log(T23/T2);
T34 = linspace(T3,T4,N);
S34 = delta_S12 + delta_S23 + n*M*cP*log(T34/T3);
T41 = linspace(T4,T1,N);
S41 = delta_S12 + delta_S23 + delta_S34 + n*M*cV*log(T41/T4);

%Check total entropy change of gas in the cycle is zero
delta_S_check = delta_S12 + delta_S23 + delta_S34 + delta_S41;

%Convert temperatures to deg Celsius and pressures to atmospheres
p1 = p1/atm;
p3 = p3/atm;
T1 = T1 - 273;
T2 = T2 - 273;
T3 = T3 - 273;
T4 = T4 - 273;
T12 = T12 - 273;
T23 = T23 - 273;
T34 = T34 - 273;
T41 = T41 - 273;

%Plot p,V diagram and save PNG
figure('name','Rectangular pV heat cycle','color',[1,1,1] )
plot([V1,V2],[p1,p1],'r-',[V2,V2],[p1,p3],'g',[V2,V1],[p3,p3],'b-',[V1,V1],[p3,p1],'k');
legend({'Isobaric expansion','Isochoric cooling',...
    'Isobaric compression','Isochoric heating'},'fontsize',fsize,'location','best')
xlabel('Volume V /m^3','fontsize',fsize);
ylabel('Pressure p /atm','fontsize',fsize);
title({'Rectangular p,V cycle',['T_1 = ',num2str(T1),'^oC, T_2 = ',...
    num2str(T2),'^oC, T_3 = ',num2str(T3),'^oC, T_4 = ',num2str(T4),'^oC'],...
    ['W = ',num2str(W/1e3),'kJ, Q_{in} = ',num2str(Q_in/1e3),...
    'kJ, Efficiency = W/Q_{in} = ',num2str(100*nu),'%']},'fontsize',fsize)
grid on;
hold on;
plot(V1,p1,'r*');
plot(V2,p1,'g*');
plot(V2,p3,'b*');
plot(V1,p3,'k*');
xlim([V1-0.1*V2,1.1*V2]);
ylim([p3-0.1*p1,1.1*p1])
set(gca,'fontsize',fsize);
print( gcf, 'rectangular pV.png','-dpng','-r300' );
cla;

%Plot T,S diagram and save PNG
plot(S12,T12,'r-',S23,T23,'g',...
    S34,T34,'b-',S41,T41,'k');
legend({'Isobaric expansion','Isochoric cooling',...
    'Isobaric compression','Isochoric heating'},'fontsize',fsize,'location','best')
xlabel('Entropy change of ideal gas /JK^{-1}','fontsize',fsize);
ylabel('Temperature /^oC','fontsize',fsize);
title({'Rectangular p,V cycle',['T_1 = ',num2str(T1),'^oC, T_2 = ',...
    num2str(T2),'^oC, T_3 = ',num2str(T3),'^oC, T_4 = ',num2str(T4),'^oC'],...
    ['W = ',num2str(W/1e3),'kJ, Q_{in} = ',num2str(Q_in/1e3),...
    'kJ, Efficiency = W/Q_{in} = ',num2str(100*nu),'%']},'fontsize',fsize)
axis auto
grid on;
hold on;
set(gca,'fontsize',fsize);
print( gcf, 'rectangular pV S vs T.png','-dpng','-r300' );
close(gcf);

%End of code